<?php

/**
 * StackBase
 * Functions that are required to access the API
 *
 * @package stackPHP
 * @author George Edison <admin@quickmediasolutions.com>, Marco Ceppi <marco.ceppi.use@gmail.com>
 */

class StackBase
{
	public function __construct( $site = NULL, $key = NULL, $cache = FALSE )
	{
		$this->site = $site;
		$this->key = $key;
		$this->api_version = '1.0';
		$this->cache = ( $cache ) ? md5($key) : FALSE;
		$this->cache_ttl = (60 * 10); // 10 Minutes
	}
	
	// Throws an error given by $id
	private function throwError($str)
	{
		// Deal with the error by throwing
		throw new Exception($str);
	}
	
	/**
	 * Fetch Data
	 * Given a URL this function will retrieve the contents and return them
	 * parsing the JSON in the process
	 * @param string $url JSON URL to retrieve.
	 * @return array|false Parsed JSON result
	 */
	private function _fetchData( $url )
	{
		$ch = curl_init();

		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HEADER, 1);
		curl_setopt($ch, CURLOPT_ENCODING, 'gzip');  // Needed by API
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

		$data = curl_exec($ch);

		// There was an error if $data is FALSE
		if( $data === FALSE )
		{
			$this->throwError("There was an error retrieving the data from the API servers.");
		}

		// If there was a 404, also an error:
		if( curl_getinfo($ch, CURLINFO_HTTP_CODE) == 404 )
		{
			$this->throwError("There was an error retrieving the data from the API servers.<br><br>The server returned a 404 response code.");
		}

		list( $header, $body ) = explode("\r\n\r\n", $data, 2);
		$header_lines = explode("\r\n", $header);
		$headers = array();

		foreach( $header_lines as $line )
		{
			if( strpos($line,':') !== false )
			{
				list( $name, $value ) = explode( ':', $line, 2 );
				$headers[trim($name)] = trim($value);
			}
		}

		$this->max = $headers['X-RateLimit-Max'];
		$this->current = $headers['X-RateLimit-Current'];

		curl_close($ch);

		return json_decode($body);
	}
	
	/**
	 * Fetch Page
	 * Given a method and params, builds a URL and fetches the data
	 * @param string $method API method to execute.
	 * @param array $params List of paramaters to execute.
	 * @return array|false 
	 */
	protected function _fetchPage($method,$params)
	{
		if(count($params))
			$str_params = '&' . implode('&',$params);
		else
			$str_params = '';

		$url = 'http://api.' . $this->site . '.com/1.0/' . $method . '?key=' . $this->key . $str_params;

		$data = $this->_fetchData($url);

		if(get_class($data) !== FALSE && property_exists($data,'error'))
			$this->throwError("There was an error retrieving the data from the API servers.<br><br><code>" . $data->error->message . "</code>");
		
		// We need to store this data
		// so that the user can access the
		// page information
		
		$this->last_request_data = $data;
		
		return $data;
	}

	/**
	 * Stats
	 * Returns useful information regarding a site
	 * 
	 * @param array $params List of paramaters to execute.
	 * 
	 * @return false|array of statistical information
	 */
	public function Stats( $params = array() )
	{
		return $this->_fetchPage('stats', $params);
	}

	/**
	 * Stack Errors
	 * Simulates an error given its code
	 * 
	 * @param int $error_id Error Code
	 * @param array $params List of paramaters to execute.
	 * 
	 * @return false|array of error information
	 */
	public function stackError( $error_id, $params = array() )
	{
		return $this->_fetchPage('errors/' . $error_id,$params);
	}
	
	/**
	 * Rate Limit Max
	 * Returns the current number of API requests
	 * @return int|false
	 */
	public function GetRateLimitCurrent()
	{
		return $this->current;
	}
	
	/**
	 * Total Items from Last Request
	 * Returns the total number of items from the subset of the last request
	 * @return int|false
	 */
	public function GetTotalItems()
	{
		return $this->last_request_data->total;
	}
	
	/**
	 * Current Page
	 * Returns the current page from the subset retrieved with the last request
	 * @return int|false
	 */
	public function GetCurrentPage()
	{
		return $this->last_request_data->page;
	}
	
	/**
	 * Total Pages
	 * Returns the total number of pages in the subset retrieved with the last request
	 * @return int|false
	 */
	public function GetTotalPages()
	{
		return $this->last_request_data->pagesize;
	}

	/**
	 * Get Cache
	 * Provide the cached result set from the key
	 * 
	 * @param string $key Typically the URL used to build the results
	 * @return false|mixed Will return false if the cache is expired or non-existent
	 */
	private function getCache( $key )
	{
		if( $this->cache !== FALSE )
		{
			$db = startCache( $this->cache );
			$current = time() - $this->cache_ttl;
			$r = $db->querySingle("SELECT updated, data FROM api_cache WHERE updated > $current", true);
			$db->close();
			
			return ( is_array($test) ) ? $r['data'] : false;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Set Cache
	 * Insert/update values for the key's cache
	 * 
	 * @param string $key Typically the URL used to build the results
	 * @param mixed $data The data to be stored for the cache's key
	 * @return bool False if unable to save cache - else will return true
	 */
	private function setCache( $key, $data )
	{
		if( $this->cache !== FALSE )
		{
			$db = startCache( $this->cache );

			$prepd_sql = $db->prepare("UPDATE api_cache SET (update = :timestmp, data = :data) WHERE key = :key");
			$prepd_sql->bindValue(':key', $key, SQLITE3_STRING);
			$prepd_sql->bindValue(':timestmp', time(), SQLITE3_INTEGER);
			$prepd_sql->bindValue(':data', $data, SQLITE3_BLOB);
			$prepd_sql->execute();
			
			$db->close();
			
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Start Cache
	 * Initiate a Cache session - create if not already existant
	 * 
	 * @param string $db_name
	 * 
	 * @return SQLite3 instance
	 */
	private function startCache( $db_name )
	{
		$create = (!file_exists("cache/$db_name.cache.db")) ? true : false;
		
		try 
		{
			$db = new SQLite3("cache/$db_name.cache.db");
		}
		catch( Exception $e ) 
		{
			throwError("Unable to initiate Caching database: $e");
		}
		
		if( $create )
		{
			$db->exec('CREATE TABLE api_cache (key STRING, updated INTEGER, data BLOB');
		}
		
		return $db;
	}
}

?>
