<?php

/**
 * UserExchange
 * Functions related to users and their statistics
 *
 * @package stackPHP
 * @author George Edison <admin@quickmediasolutions.com>, Marco Ceppi <marco.ceppi.use@gmail.com>
 */
 
require_once "StackBase.php";

class UserExchange extends StackBase
{
	public function __construct ( $site = NULL, $key = NULL )
	{
		// Initialize the parent class with
		// the API key and site
		parent::__construct( $site, $key );
	}

	/**
	 * List Users
	 * Retrieve a list of users from given site
	 * 
	 * @param array $params Array of additional parameters
	 * 
	 * @return false|array of JSON users
	 */
	public function ListUsers( $params = array() )
	{
		return $this->_fetchPage('users', $params)->users;
	}

	/**
	 * User Profile
	 * Retireve information regarding the user's public profile
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of user data
	 */
	public function UserProfile( $user_id, $params = array() )
	{
		return $this->_fetchPage('users/' . $user_id, $params)->users[0];
	}
	
	/**
	 * User's Answers
	 * Retireve all of the users answers
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of JSON answers
	 */
	public function UsersAnswers( $user_id, $params = array() )
	{
		return $this->_fetchPage('users/' . $user_id . '/answers', $params)->answers;
	}

	/**
	 * User's Badges
	 * Retireve all badges awarded to a user
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of JSON badges
	 */
	public function UsersBadges( $user_id, $params = array() )
	{
		return $this->_fetchPage('users/' . $user_id . '/badges', $params)->badges;
	}
	
	/**
	 * User's Comments
	 * Retireve all comments made by a user
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of JSON comments
	 */
	public function UsersComments( $user_id, $params = array() )
	{
		return $this->_fetchPage('users/' . $user_id . '/comments',$params)->comments;
	}

	/**
	 * User's Comments
	 * Retireve all comments made by a user
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of JSON badges
	 */
	public function UsersCommentsToUsers($user_id, $recipient_user , $params = array() )
	{
		return $this->_fetchPage('users/' . $user_id . '/' . $recipient_id, $params)->comments;
	}

	/**
	 * User's Favorites
	 * Retireve all favorites made by a user
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of JSON favorites
	 */
	public function UsersFavorites( $user_id, $params = array() )
	{
		return $this->_fetchPage('users/' . $user_id . '/favorites', $params)->questions;
	}

	/**
	 * User's Mentioned
	 * Retireve all favorites made by a user
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of mentioned JSON
	 */
	public function UsersMentioned( $user_id, $params = array() )
	{
		return $this->_fetchPage('users/' . $user_id . '/mentioned',$params)->comments;
	}

	/**
	 * User's Questions
	 * Retireve all questions asked by a user
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of User's questions from JSON
	 */
	public function UsersQuestions( $user_id, $params = array() )
	{
		return $this->_fetchPage('users/' . $user_id . '/questions', $params)->questions;
	}

	/**
	 * User's Reputation
	 * Retireve reputation of a user
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of JSON reputation
	 */
	public function UsersReputation( $user_id, $params = array() )
	{
		return $this->_fetchPage('users/' . $user_id . '/reputation', $params)->rep_changes;
	}

	/**
	 * User's Tags
	 * Retireve the tags associated with a user
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of JSON tags
	 */
	public function UsersTags( $user_id, $params = array() )
	{
		return $this->_fetchPage('users/' . $user_id . '/tags', $params)->tags;
	}

	/**
	 * User's Timeline
	 * Retireve the user's timeline
	 * 
	 * @param int $user_id Numerical user identifier for the $site
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array user's timeline
	 */
	public function UsersTimeline($id,$params=array())
	{
		return $this->_fetchPage('users/' . $id . '/timeline',$params)->user_timelines;
	}
	
	/**
	 * Moderators
	 * Retireve a list of the users who are moderators
	 * 
	 * @param array $params Parameters for the API
	 * 
	 * @return false|array of moderators
	 */
	public function Moderators( $params = array() )
	{
		return $this->_fetchPage('users/moderators',$params)->users;
	}
	
	/**
	 * User's Associations
	 * Retireve a list of all the sites a user account is associated
	 * 
	 * @param int $user_id Numerical user identifier for site
	 * 
	 * @return false|array of user's associations
	 */
	public function UsersAssociations( $user_id )
	{
		$data = $this->FetchData('http://stackauth.com/' . $this->api_version . '/users/' . $id . '/associated?key=' . $this->key);
	
		if( get_class($data) !== FALSE && property_exists($data, 'error') )
		{
			$this->_error("There was an error retrieving the data from the API servers.<br><br><code>" . $data->_error->message . "</code>");
		}
		
		return $data;
	}
}
